@@ -8,7 +8,7 @@ use Crypt::CBC;
 use MIME::Base64;
 use Exporter;
 use vars qw($VERSION @EXPORT_OK @ISA);
-$VERSION = '1.0';
+$VERSION = '1.1.1';
 @ISA = ('Exporter');
 @EXPORT_OK = qw(dmac hexdigest base64digest);
 
@@ -51,7 +51,8 @@ sub dmac
     my $subkey2 = &createsubkey($key, $iv, $cipher, 1);
 
     my $cipher1 = &createcipher($subkey1, $iv, $cipher);
-    my $mac1 = $cipher1->encrypt($data);
+    my $ciphertext = $cipher1->encrypt($data);
+    my $mac1 = substr $ciphertext, -$BLOCKSIZE;
 
     my $cipher2 = &createcipher($subkey2, $iv, $cipher);
     my $mac2 = $cipher2->encrypt($mac1);
@@ -78,7 +79,7 @@ __END__
 
 =head1 NAME
 
-Digest::DMAC 1.0
+Digest::DMAC
 
 =head1 EXAMPLE 1
 
@@ -113,33 +114,36 @@ open F, $infile;
 while (<F>) {
 
     $mac1 = &dmac($key, $cipher, $_);
-
-    $hexmac1 = hexdigest($mac1);
-
-    $base64mac1 = base64digest($mac1);
 }
 
+print hexdigest($mac1), "\n";
+
+print base64digest($mac1), "\n";
+
 =head1 DESCRIPTION
 
-This is C<Double MAC> (C<DMAC>), also known as C<Encrypted MAC>
-(C<EMAC>). Unlike C<HMAC>, which reuses an existing one-way hash
-function, such as C<MD5>, C<SHA-1> or C<RIPEMD-160>, C<DMAC> reuses
-an existing block cipher to produce a secure MAC. Using the block
-cipher, a message is encrypted in CBC mode. The last block is taken
-as the MAC of the message. For fixed-length messages, this method is
-provably secure. In reality, however, messages have arbitrary lengths,
-and this method is not secure. To make secure MACs for variable length
-messages, the last block is encrypted once again with a different key.
-The security of this construction has been proved in the paper,
-``CBC MAC for Real-Time Data Sources'' by Erez Petrank and Charles
-Rackoff. The security can be proved on the assumption that the
-underlying block cipher is pseudo-random. The performance and
-key-agility of DMAC are reasonable. DMAC is preferable for short
-messages because the block length is smaller compared to the schemes
-based on a hash function. DMAC is also chosen as one of the NESSIE
-winners for Message Authentication Codes, along with UMAC, TTMAC and
-HMAC. The current NESSIE specification chooses the C<AES> as block
-cipher.
+This is B<Double MAC> (B<DMAC>), also known as B<Encrypted MAC>
+(B<EMAC>). Unlike B<HMAC>, which reuses an existing one-way hash
+function, such as B<MD5>, B<SHA-1> or B<RIPEMD-160>, DMAC reuses an
+existing block cipher to produce a secure B<message authentication
+code> (B<MAC>).
+
+Using the block cipher, a message is encrypted in B<CBC mode>. The last
+block is taken as the MAC of the message. For fixed-length messages,
+this method is provably secure. In reality, however, messages have
+arbitrary lengths, and this method is not secure. To make secure MACs
+for variable length messages, the last block is encrypted once again
+with a different key. The security of this construction has been proved
+in the paper, ``CBC MAC for Real-Time Data Sources'' by Erez Petrank
+and Charles Rackoff. The security can be proved on the assumption that
+the underlying block cipher is pseudo-random.
+
+The performance and key-agility of DMAC are reasonable. DMAC is
+preferable for short messages because the block length is smaller
+compared to the schemes based on a hash function. DMAC is also chosen
+as one of the NESSIE winners for Message Authentication Codes, along
+with B<UMAC>, B<TTMAC> and B<HMAC>. The current NESSIE specification
+chooses the B<AES> as block cipher.
 
 Also specified in the paper by Petrank and Rackoff is the construction
 of two encryption keys from a single key. The first subkey is derived
@@ -152,11 +156,16 @@ used.
 
 =head1 PREREQUISITES
 
-The module C<Crypt::CBC> is required, plus any block cipher that is
-capable of returning its block size when queried. RC5 is not supported
-because its block size is variable. Supported block ciphers are DES,
-Blowfish, IDEA, TEA (64-bit blocks), Serpent, Rijndael (AES), Twofish,
-RC6 (128-bit blocks), and many others.
+The module B<Crypt::CBC> is required, plus any block cipher that is
+capable of returning its block size when queried. B<RC5> is not
+supported, however, because its block size is variable.
+
+B<MIME::Base64> is required if a base64 encoding of output is desired.
+
+=head1 LICENSE
+
+This program is free software; you can redistribute it or modify it
+under the terms of the GNU General Public License. 
 
 =head1 AUTHOR
 
